using System;
using System.Collections.Generic;
using Telerik.OpenAccess;

namespace FirstStepsClasses
{
    /// <summary>
    /// The persistent Customer class.
    /// The class layout has changed compared to previous steps
    /// </summary>
    [Persistent]
    public class Customer : Company
    {
        private string name;
        private int customerNo;
        private int? discountRate;

        //used to store notes about this customer
        //by employees
        private IDictionary<Employee, string> employeeSpecificNotes;

        //OpenAccess enhancer requires a no-args constructor, the
        //access rights don't matter, so it could also be private
        //The following line could also be outcommented,
        //because the compiler would create such an empty default
        //constructor for us if we have not defined any user constructor.
        public Customer() { }

        public Customer(string name, int customerNo)
            : base(name)
        {
            this.customerNo = customerNo;
        }

        public string CompanyName
        {
            get { return name; }
            set { name = value; }
        }

        public int CustomerNo
        {
            get { return customerNo; }
            set { customerNo = value; }
        }

        public int? DiscountRate
        {
            get { return discountRate; }
            set
            {
                if (discountRate.HasValue)
                    discountRate = value;
            }
        }

        public string GetNotesForEmployee(Employee empl)
        {
            if (employeeSpecificNotes == null)
            {
                return null;
            }
            else
            {
                return employeeSpecificNotes[empl];
            }
        }

        public void SetNotesForEmployee(Employee empl, string notes)
        {
            if (employeeSpecificNotes == null)
            {
                employeeSpecificNotes = new Dictionary<Employee, string>();
            }
            else
            {
                //the old value has to be removed first,
                //if there is no value contained for empl, then it is a no-op
                employeeSpecificNotes.Remove(empl);
            }
            employeeSpecificNotes[empl] = notes;
        }

        public override string ToString()
        {
            return "[" + customerNo + "] " + base.ToString();
        }

    }
}
