using System;
using System.Collections;
using FirstStepsClasses;
using Telerik.OpenAccess;
using Telerik.OpenAccess.MetaData;
using System.ComponentModel;

namespace Utilities
{
    /// <summary>
    /// This helper class populates the ExampleBase.
    /// This logic is contained in a separate Class so it can
    /// be resused in later steps.
    /// </summary>
    public class FillDB
    {
        private static bool databaseUpDated = false;
        public static void Fill(Database db)
        {

            CreateUpdateDataBase(db);
            if (!databaseUpDated)
            {
                IObjectScope scope = db.GetObjectScope();
                FillPersonsTable(scope);
                scope.Transaction.Begin();

                IQueryResult existingObjects = scope.GetOqlQuery(
                        "SELECT * FROM ProductExtent AS p WHERE p.name == 'The Road Ahead'").Execute();
                if (existingObjects.Count == 0)
                {
                    IList objects = CreateObjects();
                    IEnumerator enumerator = objects.GetEnumerator();
                    scope.Add(enumerator);
                }
                existingObjects.Dispose();

                scope.Transaction.Commit();
            }
        }

        private static void CreateUpdateDataBase(Database db)
        {
            db.Dispose();
            db = Database.Get("DBConnFirstSteps");
            if (!db.GetSchemaHandler().DatabaseExists())
            {
                db.GetSchemaHandler().CreateDatabase();
            }
            string script = db.GetSchemaHandler().CreateUpdateDDLScript(null);
            if (!String.IsNullOrEmpty(script))
            {
                db.GetSchemaHandler().ExecuteDDLScript(script);
            }
            else
            {
                databaseUpDated = true;
            }
        }

        private static void FillPersonsTable(IObjectScope scope)
        {
            QueryResultList<Person> persons = scope.GetOqlQuery<Person>().ExecuteList();
            if (persons.Count == 0)
            {

                Person person1 = new Person();
                Person person2 = new Person();
                Person person3 = new Person();
                Person person4 = new Person();
                Person person5 = new Person();
                scope.Transaction.Begin();
                scope.Add(person1);
                scope.Add(person2);
                scope.Add(person3);
                scope.Add(person4);
                scope.Add(person5);
                SetPropertiesToPerson(scope, person1, "John", "Smith", 32, "High Avenue 582");
                SetPropertiesToPerson(scope, person2, "Ana", "Jackson", 20, "Wall Street 41");
                SetPropertiesToPerson(scope, person3, "Steven", "Williams", 61, "Rocky Mountains");
                SetPropertiesToPerson(scope, person4, "Maria", "Jansen", 25, "Street Of Dreams");
                SetPropertiesToPerson(scope, person5, "Lincoln", "Murphy", 38, "Five Points");
                scope.Transaction.Commit();
            }
        }

        private static void SetPropertiesToPerson(IObjectScope scope, Person person, string firstname, string lastname,
            int age, string address)
        {
            IPersistentTypeDescriptor ptd = scope.PersistentMetaData.GetPersistentTypeDescriptor(typeof(Person));
            PropertyDescriptorCollection pdc = ptd.GetProperties();
            pdc["firstName"].SetValue(person, firstname);
            pdc["lastName"].SetValue(person, lastname);
            pdc["age"].SetValue(person, age);
            pdc["address"].SetValue(person, address);
        }

        public static void Delete(IObjectScope scope)
        {
            scope.Transaction.Begin();

            /* todo_api foreach(object o in scope.GetExtent(typeof(object)))
            {
                scope.Remove(o);
                }*/

            scope.Transaction.Commit();
        }

        private static IList CreateObjects()
        {
            //create some products
            Product[] products = new Product[8];
            products[0] = new Book(1001, "The Road Ahead", 9.90f, "Bill Gates");
            products[1] = new Book(1002, "Programming Windows", 49.90f, "Charles Petzold");
            products[2] = new Video(1003, "The Life of Brian", 19.90f, Rating.R, "Terry Jones", 89);
            products[3] = new Video(1004, "A Fish called Wanda", 18.90f, Rating.R, "John Cleese", 103);
            products[4] = new CdRom(1005, "Windows XP Home Edition", 99.90f, 600, FileSystem.ISO9660);
            products[5] = new CdRom(1006, "Windows XP Prof. Edition", 250.00f, 600, FileSystem.ISO9660);
            products[6] = new MusicCd(1005, "All That You Can't Leave Behind", 17.90f, 11, "U2", 55);
            products[7] = new MusicCd(1006, "Meteora", 23.90f, 13, "Linkin Park", 60);

            //create some categories and link products with them
            Category[] categories = new Category[2];
            categories[0] = new Category("Amusement");
            categories[1] = new Category("Technical");
            products[0].Category = categories[0];
            products[1].Category = categories[1];
            products[2].Category = categories[0];
            products[3].Category = categories[0];
            products[4].Category = categories[1];
            products[5].Category = categories[1];
            products[6].Category = categories[0];
            products[7].Category = categories[0];

            //create some suppliers and link products with them
            //the supplier for the DVDs in unknown
            Supplier[] suppliers = new Supplier[5];
            suppliers[0] = new Supplier("Universal", 1);
            suppliers[0].Addresses.Add(new Address("Universal Blvd.", 77777, "Los Angeles"));
            suppliers[0].Addresses.Add(new Address("Potsdamer Platz", 10000, "Berlin"));
            suppliers[1] = new Supplier("Langenscheid-Longman", 2);
            suppliers[2] = new Supplier("Warner Music", 3);
            suppliers[2].Addresses.Add(new Address("TowerBridge", 20000, "London"));
            suppliers[3] = new Supplier("Microsoft Press", 4);
            suppliers[4] = new Supplier("Microsoft", 4);
            products[0].Supplier = suppliers[1];
            products[1].Supplier = suppliers[3];
            products[4].Supplier = suppliers[4];
            products[5].Supplier = suppliers[4];
            products[6].Supplier = suppliers[0];
            products[7].Supplier = suppliers[2];

            //create some employees
            Employee[] employees = new Employee[3];
            employees[0] = new Employee(
                "James",
                "Smith",
                new Address("Sunset Blvd. 3", 12345, "Chicago"),
                new DateTime(1975, 12, 10));
            employees[1] = new Employee(
                "Peter",
                "Miller",
                new Address("Mainstreet 5", 12350, "Chicago"),
                new DateTime(1960, 7, 1));
            employees[2] = new Employee(
                "John",
                "Davis",
                new Address("Mainstreet 30", 12350, "Chicago"),
                new DateTime(1970, 1, 30));
            employees[0].Boss = employees[1];
            employees[2].Boss = employees[1];





            //create some customers
            Customer[] customers = new Customer[3];
            customers[0] = new Customer("The Music Store", 4567);
            customers[1] = new Customer("DVD Superstore", 6352);
            customers[2] = new Customer("Press Centre", 8876);

            //create some shippers
            Shipper[] shippers = new Shipper[2];
            shippers[0] = new Shipper("UPS", 1);
            shippers[1] = new Shipper("DHL", 2);

            //create some random orders
            ArrayList orders = new ArrayList();
            Random rand = new Random();
            for (int i = 0; i < 200; i++)
            {
                Order order = new Order();
                order.Customer = customers[rand.Next(customers.Length)];
                order.EmployeeInCharge = employees[rand.Next(employees.Length)];
                order.OrderNo = 1000 + i;
                order.OrderDate = new DateTime(1999 + rand.Next(5),
                                                rand.Next(12) + 1,
                                                rand.Next(28) + 1);
                order.ShippedDate = order.OrderDate.AddDays(rand.Next(5));
                order.Shipper = shippers[rand.Next(shippers.Length)];
                for (int j = rand.Next(10) + 1; j > 0; j--)
                {
                    Product product = products[rand.Next(products.Length)];
                    order.Details.Add(new OrderDetail(
                                            product.UnitPrice,
                                            rand.Next(4) + 1,
                                            product));
                }
                orders.Add(order);
            }

            //it can happen, that not all objects were referenced, thus,
            //perhaps not all preconstructed objects are stored,
            //this could be avoided by explicitly "Add()"ing them

            orders.AddRange(employees);
            orders.AddRange(categories);

            return orders;
        }
    }
}
