using System;
using System.Collections.Generic;
using Telerik.OpenAccess;

namespace FirstStepsClasses
{
	/// <summary>
	/// The persistent Order class.
	/// The class layout has changed compared with previous steps.
	/// </summary>
	[Persistent]
	public class Order
	{
        [FetchField("Invoice Fetch Group", Path="name")]
        private Customer customer;
        [FetchField("Invoice Fetch Group")]
		private int orderNo;
		private DateTime orderDate;
		private DateTime shippedDate;
		private Employee employeeInCharge;
		private Shipper shipper;

        [FetchField("Invoice Fetch Group", Depth = 3)]
        private IList<OrderDetail> details;
		
		public Order() {}

        public Order(Customer _customer)
        {
            this.customer = _customer;
        }

		public Customer Customer
		{
			get { return customer; }
			set { customer = value; }
		}

		public int OrderNo
		{
			get { return orderNo; }
			set { orderNo = value; }
		}

		public DateTime OrderDate
		{
			get { return orderDate; }
			set { orderDate = value; }
		}

		public DateTime ShippedDate
		{
			get { return shippedDate; }
			set { shippedDate = value; }
		}

		public IList<OrderDetail> Details
		{
			get 
			{
				if( details == null )
				{
					details = new List<OrderDetail>();
				}
				return details;
			}
		}

		public Employee EmployeeInCharge
		{
			get { return employeeInCharge; }
			set { employeeInCharge = value; }
		}

		public Shipper Shipper
		{
			get { return shipper; }
			set { shipper = value; }
		}

		public override string ToString()
		{
			System.Text.StringBuilder sbuilder = new System.Text.StringBuilder();
			sbuilder.Append("Customer:\r\n");
			sbuilder.Append(customer);
			sbuilder.Append("\r\n\r\nOrderDate: " + orderDate.ToShortDateString());
			sbuilder.Append("\r\nShippedDate: " + shippedDate.ToShortDateString());
			sbuilder.Append("\r\n\r\nDetails:\r\n");
			foreach(OrderDetail od in Details)
			{
				sbuilder.Append(od);
				sbuilder.Append("\r\n");
			}
			return sbuilder.ToString();
		}


	}
}
