using System;
using Telerik.OpenAccess;

namespace FirstStepsClasses
{
	//it is not necessary or allowed to declare an enum as persistent
	//it is allowed, to use an enum from another library as persistent member,
	//e.g. System.IO.FileMode
	public enum Rating {G, PG, PG13, R, NC17};
	
	/// <summary>
	/// The persistent Video class.
	/// </summary>
	[Persistent]
	public class Video : Product
	{
		private Rating rating;
		//just a string to keep the datamodel simple
		private string director;
		private int? playLength;
		
		//no-args constructor required by teh OpenAccess runtime system
		private Video() {}
		
		public Video( int productNo, 
					  string name, 
					  float unitPrice, 
					  Rating rating,
					  string director,
					  int playLength ) :
			base(productNo, name, unitPrice)
		{
			this.rating = rating;
			this.director = director;
			this.playLength = playLength;
		}

		public Rating Rating
		{
			get { return rating; }
			set { rating = value; }
		}

		public string Director
		{
			get { return director; }
			set { director = value; }
		}

		public int? PlayLength
		{
			get { return playLength; }
			set { playLength = value; }
		}

		public override string ToString()
		{
			return base.ToString() + " [Rated: " + rating + "]";
		}

	}
}
